// Area.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <libfuture.h>
#include <Utility.h>
#include <sys/param.h>

#include "Area.h"
#include "Terrain.h"
#include "Chunk.h"


// statics
areas_t Area::m_areas;


Area::Area(Database& db,long num) : Object(db)
{
	m_area = Area::GetArea(db, num);
	if (Exists())
	{
		VerifyTerrain();
	}
}


Area::Area(Database& db,const std::string& name) : Object(db)
{
	m_area = Area::GetArea(db, name);
	if (Exists())
	{
		VerifyTerrain();
	}
}


Area::~Area()
{
}



bool Area::Exists()
{
	if (m_area && m_area -> num)
		return true;
	return false;
}


void Area::Save()
{
	m_area -> save();
	VerifyTerrain();
}


int Area::GetWidth()
{
	return m_area -> width;
}


int Area::GetHeight()
{
	return m_area -> height;
}


long Area::GetNum()
{
	return m_area -> num;
}


void Area::SetWidth(int w)
{
	m_area -> width = w;
}


void Area::SetHeight(int h)
{
	m_area -> height = h;
}


void Area::SetName(const std::string& name)
{
	m_area -> name = name;
}


const std::string& Area::GetName()
{
	return m_area -> name;
}


void Area::VerifyTerrain()
{
	{
		Terrain t(m_db, *this, 'P');
		if (!t.Exists())
		{
			t.SetColor("&m");
			t.SetName("Portal");
			t.Save();
		}
	}
	{
		Terrain t(m_db, *this, 'U');
		if (!t.Exists())
		{
			t.SetColor("&m");
			t.SetName("Stairs up");
			t.Save();
		}
	}
	{
		Terrain t(m_db, *this, 'D');
		if (!t.Exists())
		{
			t.SetColor("&m");
			t.SetName("Stairs down");
			t.Save();
		}
	}
}


db::Area *Area::GetArea(Database& db,long num)
{
	for (areas_t::iterator it = m_areas.begin(); it != m_areas.end(); it++)
	{
		db::Area *p = *it;
		if (p -> num == num)
			return p;
	}
	db::Area *p = new db::Area(db, num);
	m_areas.push_back(p);
	return p;
}


db::Area *Area::GetArea(Database& db,const std::string& name)
{
	for (areas_t::iterator it = m_areas.begin(); it != m_areas.end(); it++)
	{
		db::Area *p = *it;
		if (p -> name == name)
			return p;
	}
	db::Area *p = new db::Area(db, name);
	p -> name = name;
	m_areas.push_back(p);
	return p;
}


void Area::GetAreaBorders(int ch_z,int& xmin,int& xmax,int &ymin,int &ymax)
{
	xmin = ymin = 1000000;
	xmax = ymax = 0;
	Query q(m_db);
	q.get_result("select * from chunk where area=" + Utility::l2string(m_area -> num) +
		" and z=" + Utility::l2string(ch_z) );
	while (q.fetch_row())
	{
		db::Chunk chunk(&m_db, &q);
		xmin = MIN(xmin, chunk.x);
		xmax = MAX(xmax, chunk.x);
		ymin = MIN(ymin, chunk.y);
		ymax = MAX(ymax, chunk.y);
	}
	q.free_result();
}


