// Chunk.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <libfuture.h>
#include <Utility.h>

#include "Chunk.h"
#include "InSocket.h"
#include "Player.h"
#include "FutureHandler.h"
#include "Area.h"
#include "Terrain.h"
#include "Chunkinfo.h"
#include "Portal.h"


// statics
chunks_t Chunk::m_chunks;


Chunk::Chunk(Database& db,Area& a,long num) : Object(db), m_area(a)
, m_chunk(Chunk::GetChunk(db, a, num))
{
	GetInfo();
}


Chunk::Chunk(Database& db,Area& a,int x,int y,int z) : Object(db), m_area(a)
, m_chunk(Chunk::GetChunk(db, a, x, y, z))
{
	GetInfo();
}


Chunk::~Chunk()
{
}


void Chunk::Init(InSocket *from,int x,int y,int z,char tchar)
{
	{
		db::Chunk chunk(&m_db);
		chunk.area = m_area.GetNum();
		chunk.x = x;
		chunk.y = y;
		chunk.z = z;
		if (from)
		{
			Database& db = from -> GetDatabase();
			Player pl(db, from -> GetAccountName());
			chunk.created_by = pl.GetNum();
		}
		else
		{
			chunk.created_by = 0;
		}
		// next line will crash when initing a new world
		chunk.created_date = static_cast<FutureHandler&>(from -> Handler()).GetDateTime();
		while (chunk.terrain.size() < (size_t)m_area.GetWidth() * m_area.GetHeight())
			chunk.terrain += tchar;
		chunk.save();
	}
	Chunk::RemoveObject(m_area, x, y, z);
	m_chunk = Chunk::GetChunk(m_db, m_area, x, y, z);

	Chunk cnorth(m_db, m_area, x, y + 1, z);
	if (cnorth.Exists())
	{
		for (int x = 0; x < m_area.GetWidth(); x++)
			Set(x, 0, cnorth.Get(x, m_area.GetHeight() - 1));
	}

	Chunk csouth(m_db, m_area, x, y - 1, z);
	if (csouth.Exists())
	{
		for (int x = 0; x < m_area.GetWidth(); x++)
			Set(x, m_area.GetHeight() - 1, csouth.Get(x, 0));
	}

	Chunk ceast(m_db, m_area, x + 1, y, z);
	if (ceast.Exists())
	{
		for (int y = 0; y < m_area.GetHeight(); y++)
			Set(m_area.GetWidth() - 1, y, ceast.Get(0, y));
	}

	Chunk cwest(m_db, m_area, x - 1, y, z);
	if (cwest.Exists())
	{
		for (int y = 0; y < m_area.GetHeight(); y++)
			Set(0, y, cwest.Get(m_area.GetWidth() - 1, y));
	}
}


void Chunk::Set(int x,int y,char tchar)
{
	if (m_chunk)
		m_chunk -> terrain[x + y * m_area.GetWidth()] = tchar;
}


char Chunk::Get(int x,int y)
{
	if (m_chunk)
		return m_chunk -> terrain[x + y * m_area.GetWidth()];
	return ' ';
}


bool Chunk::Exists()
{
	if (m_chunk && m_chunk -> num)
		return true;
	return false;
}


void Chunk::Save()
{
	if (m_chunk)
		m_chunk -> save();
}


long Chunk::GetNum()
{
	if (m_chunk)
		return m_chunk -> num;
	return 0;
}


void Chunk::Fill(int x,int y,char fromchar,char tochar)
{
	Set(x,y,tochar);
	if (y > 0 && Get(x,y - 1) == fromchar)
		Fill(x, y - 1, fromchar, tochar);
	if (y < m_area.GetHeight() - 1 && Get(x,y + 1) == fromchar)
		Fill(x, y + 1, fromchar, tochar);
	if (x < m_area.GetWidth() - 1 && Get(x + 1,y) == fromchar)
		Fill(x + 1, y, fromchar, tochar);
	if (x > 0 && Get(x - 1,y) == fromchar)
		Fill(x - 1, y, fromchar, tochar);
}


db::Chunk *Chunk::GetChunk(Database& db,Area& area,int x,int y,int z)
{
	std::list<db::Chunk *>& ref = m_chunks[area.GetNum()];
	for (std::list<db::Chunk *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunk *p = *it;
		if (p -> x == x && p -> y == y && p -> z == z)
		{
			return p;
		}
	}
	db::Chunk *p = new db::Chunk(db, area.GetNum(), x, y, z);
	p -> area = area.GetNum();
	p -> x = x;
	p -> y = y;
	p -> z = z;
	ref.push_back(p);
	return p;
}


db::Chunk *Chunk::GetChunk(Database& db,Area& area,long num)
{
	std::list<db::Chunk *>& ref = m_chunks[area.GetNum()];
	for (std::list<db::Chunk *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunk *p = *it;
		if (p -> num == num)
		{
			return p;
		}
	}
	db::Chunk *p = new db::Chunk(db, num);
	p -> area = area.GetNum();
	ref.push_back(p);
	return p;
}


void Chunk::RemoveObject(Area& area,int x,int y,int z)
{
	std::list<db::Chunk *>& ref = m_chunks[area.GetNum()];
	for (std::list<db::Chunk *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunk *p = *it;
		if (p -> x == x && p -> y == y && p -> z == z)
		{
			delete p;
			ref.erase(it);
			return;
		}
	}
}


int Chunk::GetX()
{
	return m_chunk -> x;
}


int Chunk::GetY()
{
	return m_chunk -> y;
}


int Chunk::GetZ()
{
	return m_chunk -> z;
}


void Chunk::Edge(Terrain& t,int width)
{
	if (width < m_area.GetWidth() && width < m_area.GetHeight())
	{
		for (int w = 0; w < width; w++)
		{
			for (int x = 0; x < m_area.GetWidth(); x++)
			{
				Set(x, w, t.GetChar());
				Set(x, m_area.GetHeight() - w - 1, t.GetChar());
			}
			for (int y = 0; y < m_area.GetHeight(); y++)
			{
				Set(w, y, t.GetChar());
				Set(m_area.GetWidth() - w - 1, y, t.GetChar());
			}
		}
	}
}


void Chunk::GetInfo()
{
	if (!Exists())
		return;
	Query q(m_db);
	q.get_result("select * from chunkinfo where chunk=" + Utility::l2string(GetNum()));
	while (q.fetch_row())
	{
		db::Chunkinfo info(&m_db, &q);
		Chunkinfo x(m_db, *this, info.x, info.y, (Chunkinfo::infotype_t)info.type);
	}
	q.free_result();
}


void Chunk::AddPortal(int x,int y,Area& to_area,Chunk& to_chunk,int to_x,int to_y)
{
	Portal info(m_db, *this);
	info.SetX(x);
	info.SetY(y);
	info.SetType(Chunkinfo::TYPE_PORTAL);
	info.SetToArea(to_area.GetNum());
	info.SetToChunk(to_chunk.GetNum());
	info.SetToX(to_x);
	info.SetToY(to_y);
	info.Save();
	Set(x, y, 'P');
	Save();
}


void Chunk::Show(InSocket *from)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	std::list<db::Chunkinfo *>& ref = Chunkinfo::GetChunkinfo(*this);
	for (std::list<db::Chunkinfo *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunkinfo *p = *it;
		if (p -> x == pl.GetChunkX() && p -> y == pl.GetChunkY())
		{
			Chunkinfo *info = Chunkinfo::GetInstance(db, *this, p -> x, p -> y, (Chunkinfo::infotype_t)p -> type);
			info -> Display(from);
			delete info;
		}
	}
}


