// Chunk.h
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _CHUNK_H
#define _CHUNK_H

#include <map>
#include <list>
#include "Object.h"


class InSocket;
namespace db { class Chunk; }
class Database;
class Area;
class Terrain;

typedef std::map<long,std::list<db::Chunk *> > chunks_t;

/** Database table Chunk
	\ingroup db */
class Chunk : public Object
{
public:
	Chunk(Database& ,Area& ,long num);
	Chunk(Database& ,Area& ,int x,int y,int z = 0);
	~Chunk();

	void Init(InSocket *,int x,int y,int z,char tchar);

	void Set(int x,int y,char tchar);
	char Get(int x,int y);
	void Fill(int x,int y,char fromchar,char tochar);
	void Edge(Terrain& t,int width);

	bool Exists();
	int GetX();
	int GetY();
	int GetZ();

	void Save();
	long GetNum();

static	void RemoveObject(Area& ,int x,int y,int z);

	void AddPortal(int x,int y,Area& to_area,Chunk& to_chunk,int to_x,int to_y);
	void Show(InSocket *);

private:
static	db::Chunk *GetChunk(Database&,Area&,int x,int y,int z = 0);
static	db::Chunk *GetChunk(Database&,Area&,long num);
	void GetInfo();
	Area& m_area;
	db::Chunk *m_chunk;
static	chunks_t m_chunks;
};




#endif // _CHUNK_H
