// Chunkinfo.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <libfuture.h>

#include "Chunkinfo.h"
#include "Chunk.h"
#include "InSocket.h"
#include "Player.h"
#include "Portal.h"
#include "RoomDescription.h"


// statics
chunkinfos_t Chunkinfo::m_cache;


Chunkinfo::Chunkinfo(Database& db,Chunk& chunk) : Object(db), m_chunk(chunk)
{
	m_chunkinfo = Chunkinfo::GetChunkinfo(db, chunk);
}


Chunkinfo::Chunkinfo(Database& db,Chunk& chunk,int x,int y,infotype_t type) : Object(db), m_chunk(chunk)
{
	m_chunkinfo = Chunkinfo::GetChunkinfo(db, chunk, x, y, type);
}


Chunkinfo::~Chunkinfo()
{
}


void Chunkinfo::Save()
{
	m_chunkinfo -> save();
}


bool Chunkinfo::Exists()
{
	if (m_chunkinfo && m_chunkinfo -> num)
		return true;
	return false;
}


long Chunkinfo::GetNum()
{
	return m_chunkinfo -> num;
}


db::Chunkinfo *Chunkinfo::GetChunkinfo(Database& db,Chunk& chunk)
{
	std::list<db::Chunkinfo *>& ref = m_cache[chunk.GetNum()];
	db::Chunkinfo *p = new db::Chunkinfo(&db);
	p -> chunk = chunk.GetNum();
	ref.push_back(p);
	return p;
}


db::Chunkinfo *Chunkinfo::GetChunkinfo(Database& db,Chunk& chunk,long num)
{
	std::list<db::Chunkinfo *>& ref = m_cache[chunk.GetNum()];
	for (std::list<db::Chunkinfo *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunkinfo *p = *it;
		if (p -> num == num)
			return p;
	}
	db::Chunkinfo *p = new db::Chunkinfo(db, num);
	p -> chunk = chunk.GetNum();
	ref.push_back(p);
	return p;
}


void Chunkinfo::SetType(infotype_t x)
{
	m_chunkinfo -> type = x;
}


void Chunkinfo::SetData(const std::string& x)
{
	m_chunkinfo -> data = x;
}


Chunkinfo::infotype_t Chunkinfo::GetType()
{
	return (infotype_t)m_chunkinfo -> type;
}


const std::string& Chunkinfo::GetData()
{
	return m_chunkinfo -> data;
}


void Chunkinfo::SetX(int x)
{
	m_chunkinfo -> x = x;
}


void Chunkinfo::SetY(int x)
{
	m_chunkinfo -> y = x;
}


int Chunkinfo::GetX()
{
	return m_chunkinfo -> x;
}


int Chunkinfo::GetY()
{
	return m_chunkinfo -> y;
}


db::Chunkinfo *Chunkinfo::GetChunkinfo(Database& db,Chunk& chunk,int x,int y,infotype_t type)
{
	std::list<db::Chunkinfo *>& ref = m_cache[chunk.GetNum()];
	for (std::list<db::Chunkinfo *>::iterator it = ref.begin(); it != ref.end(); it++)
	{
		db::Chunkinfo *p = *it;
		if (p -> x == x && p -> y == y && p -> type == type)
		{
			return p;
		}
	}
	db::Chunkinfo *p = new db::Chunkinfo(db, chunk.GetNum(), type, x, y);
	p -> chunk = chunk.GetNum();
	p -> type = type;
	p -> x = x;
	p -> y = y;
	ref.push_back(p);
	return p;
}


std::list<db::Chunkinfo *>& Chunkinfo::GetChunkinfo(Chunk& chunk)
{
	return m_cache[chunk.GetNum()];
}


Chunkinfo *Chunkinfo::GetInstance(Database& db,Chunk& chunk,int x,int y,infotype_t type)
{
	switch (type)
	{
	case TYPE_PORTAL:
		return new Portal(db, chunk, x, y);
	case TYPE_ROOM_DESCRIPTION:
		return new RoomDescription(db, chunk, x, y);
	}
	return NULL;
}


