// Chunkinfo.h
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _CHUNKINFO_H
#define _CHUNKINFO_H

#include <map>
#include <list>
#include "Object.h"


class Database;
class Chunk;
namespace db { class Chunkinfo; }
class InSocket;

typedef std::map<long,std::list<db::Chunkinfo *> > chunkinfos_t;

/** Database table Chunkinfo
	\ingroup db */
class Chunkinfo : public Object
{
public:
	typedef enum // bits, please
	{
		TYPE_PORTAL = 1,
		TYPE_ROOM_DESCRIPTION = 2,
	} infotype_t;
public:
	/** Create and save empty chunkinfo. */
	Chunkinfo(Database& db,Chunk& chunk);
	/** Get location specific chunkinfo. */
	Chunkinfo(Database& db,Chunk& chunk,int x,int y,infotype_t type);
	~Chunkinfo();

	void Save();
	bool Exists();
	long GetNum();

	void SetX(int);
	void SetY(int);
	int GetX();
	int GetY();
	void SetType(infotype_t);
	void SetData(const std::string&);
	infotype_t GetType();
	const std::string& GetData();

	virtual void Display(InSocket *) {}

static	std::list<db::Chunkinfo *>& GetChunkinfo(Chunk& );
static	Chunkinfo *GetInstance(Database&,Chunk&,int x,int y,infotype_t type);

protected:
	db::Chunkinfo *m_chunkinfo;

private:
static	db::Chunkinfo *GetChunkinfo(Database& db,Chunk& chunk);
static	db::Chunkinfo *GetChunkinfo(Database& db,Chunk& chunk,long num);
static	db::Chunkinfo *GetChunkinfo(Database& db,Chunk& chunk,int x,int y,infotype_t type);
	Chunk& m_chunk;
static	chunkinfos_t m_cache;
};




#endif // _CHUNKINFO_H
