// Command.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//#include <stdio.h>

#include "Command.h"
#include "InSocket.h"
#include "Player.h"


// statics
std::map<char,Command *> Command::m_ccs;


Command::Command() : m_cmd(""),m_cc(0),m_need_wiz(false)
{
}


Command::Command(const std::string& cmd,bool w) : m_cmd(cmd),m_cc(0),m_need_wiz(w)
{
}


Command::Command(const std::string& cmd,char cc,bool w) : m_cmd(cmd),m_cc(cc),m_need_wiz(w)
{
	if (m_ccs[cc] != NULL)
	{
		fprintf(stderr, "Two commands using the same char: %c - exiting...\n", cc);
		exit(-1);
	}
	m_ccs[cc] = this;
}


Command::~Command()
{
}


int Command::Match(InSocket *from,const std::string& cmd,const std::string& params)
{
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	if (m_need_wiz && !pl.IsWizard())
		return 0;
	if (cmd.size() > m_cmd.size())
		return 0;
	if (!strcasecmp(cmd.c_str(), m_cmd.c_str()))
		return 2;
	if (!strncasecmp(cmd.c_str(), m_cmd.c_str(), cmd.size()))
		return 1;
	return 0;
}


Command *Command::GetCommandFromChar(char cc)
{
	for (std::map<char,Command *>::iterator it = m_ccs.begin(); it != m_ccs.end(); it++)
	{
		if ((*it).first == cc)
			return (*it).second;
	}
	return NULL;
}


void Command::OnLine(InSocket *from,const std::string& line,int state)
{
	from -> SetPrompt();
}


void Command::AddArgument(const std::string& arg,bool wiz)
{
	m_args[arg] = wiz;
}


bool Command::CheckArgument(InSocket *from,const std::string& arg,std::string& complete)
{
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	bool wiz = pl.IsWizard();
	if (!m_args.size())
	{
		complete = arg;
		return true;
	}
	std::string found;
	int count = 0;
	for (strbool_m::iterator it = m_args.begin(); it != m_args.end(); it++)
	{
		std::string str = (*it).first;
		bool need_wiz = (*it).second;
		if (!need_wiz || wiz)
		{
			if (str.size() >= arg.size() &&
				!strncasecmp(str.substr(0, arg.size()).c_str(), arg.c_str(), arg.size()))
			{
				found = str;
				count++;
			}
		}
	}
	if (count == 1)
	{
		complete = found;
		return true;
	}
	if (!count)
	{
		from -> Send("Please specify:\n");
		for (strbool_m::iterator it = m_args.begin(); it != m_args.end(); it++)
		{
			std::string str = (*it).first;
			bool need_wiz = (*it).second;
			if (!need_wiz || wiz)
			{
				from -> Send("  " + str + "\n");
			}
		}
	}
	else
	{
		from -> Send("Please be more specific:\n");
		for (strbool_m::iterator it = m_args.begin(); it != m_args.end(); it++)
		{
			std::string str = (*it).first;
			bool need_wiz = (*it).second;
			if (!need_wiz || wiz)
			{
				if (str.size() >= arg.size() &&
					!strncasecmp(str.substr(0, arg.size()).c_str(), arg.c_str(), arg.size()))
				{
					from -> Send("  " + str + "\n");
				}
			}
		}
	}
	from -> SetPrompt();
	return false;
}


