// Command.h
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _COMMAND_H
#define _COMMAND_H

#include <string>
#include <map>


class InSocket;
class Parse;

typedef std::map<std::string,bool> strbool_m;

/** \defgroup command Commands */
/** Command base class.
	\ingroup command */
class Command
{
public:
	Command();
	Command(const std::string& cmd,bool need_wiz = false);
	Command(const std::string& cmd,char cc,bool need_wiz = false);
	virtual ~Command();

	void AddArgument(const std::string& ,bool wiz = false);
	bool CheckArgument(InSocket *,const std::string&,std::string& );
	strbool_m& GetArguments() { return m_args; }

	/** 0-no match,1-partial match,2-exact match */
	int Match(InSocket *,const std::string& cmd,const std::string& params);
	/** Execute command. */
	virtual void Execute(InSocket *,const std::string& arg,Parse& ) = 0;
	/** Additional callback, reply to command prompt.
		\sa InSocket::SetPrompt */
	virtual void OnLine(InSocket *,const std::string& line,int state);
	/** Get command category. */
	virtual std::string Category() = 0;
	/** Command description. */
	virtual std::string Description() { return ""; }

	const std::string& GetCommand() { return m_cmd; }
	char GetCommandChar() { return m_cc; }

static	Command *GetCommandFromChar(char cc);

	bool NeedWiz() { return m_need_wiz; }

private:
	std::string m_cmd;
	char m_cc;
static	std::map<char,Command *> m_ccs;
	strbool_m m_args;
	bool m_need_wiz;
};




#endif // _COMMAND_H
