// Create.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <Parse.h>
#include <Utility.h>

#include "Create.h"
#include "InSocket.h"
#include "Player.h"
#include "Terrain.h"
#include "Area.h"
#include "Chunk.h"
#include "Editor.h"
#include "Race.h"


void Create::Execute(InSocket *from,const std::string& param,Parse& pa)
{
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	if (param == "terrain" && pl.IsWizard() )
	{
		Database& db = from -> GetDatabase();
		Area area(db, pl.GetArea());
		std::string rest = pa.getrest();
		if (!rest.size())
		{
			from -> SetPrompt(this, 0, "Enter terrain character to create/modify: ");
		}
		else
		{
			OnLine(from, rest, 0);
		}
	}
	else
	if (param == "area")
	{
		std::string rest = pa.getrest();
		if (!rest.size())
		{
			from -> SetPrompt(this, 10, "Name of new area? ");
		}
		else
		{
			OnLine(from, rest, 10);
		}
	}
	else
	if (param == "description")
	{
		Editor *e = new Editor(from, Editor::EDIT_ROOM_DESCRIPTION);
		e -> Init();
	}
	else
	if (param == "race")
	{
		int n = Race::DisplayRaces(from);
		from -> SetPrompt(this, 20, "Enter a race to edit (1-" + Utility::l2string(n) + ") or 0 to create: ");
	}
	else
	{
		from -> Send("Create what?\n");
		from -> SetPrompt();
	}
}


void Create::OnLine(InSocket *from,const std::string& line,int state)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	switch (state)
	{
	case 0:
		if (!line.size())
		{
			from -> SetPrompt();
		}
		else
		if (line.size() == 1)
		{
			Terrain t(db, area, line[0]);
			from -> Send("Terrain character: " + line + "\n");
			from -> SetCommandData(line);
			if (t.Exists())
			{
				from -> SetPrompt(this, 1, "Modify (Y/n)? ");
			}
			else
			{
				from -> SetPrompt(this, 1, "Create (y/N)? ");
			}
		}
		else
		{
			from -> Send("Only one character allowed.\n");
			from -> SetPrompt();
		}
		break;
	case 1:
		{
			std::string tchar = from -> GetCommandData();
			Terrain t(db, area, tchar[0]);
			if (t.Exists())
			{
				if (!line.size() || (line[0] != 'n' && line[0] != 'N'))
				{
					from -> SetPrompt(this, 2, "Color string (" + t.GetColor() + ")? ", false);
				}
				else
				{
					from -> SetPrompt();
				}
			}
			else
			{
				if (line.size() && (line[0] == 'y' || line[0] == 'Y'))
				{
					t.SetChar(tchar[0]);
					t.Save();
					from -> SetPrompt(this, 2, "Color string? ");
				}
				else
				{
					from -> SetPrompt();
				}
			}
		}
		break;
	case 2:
		{
			std::string tchar = from -> GetCommandData();
			Terrain t(db, area, tchar[0]);
			if (line.size())
			{
				t.SetColor(line);
				t.Save();
			}
			if (t.GetName().size())
				from -> SetPrompt(this, 3, "Terrain name (" + t.GetName() + ")? ");
			else
				from -> SetPrompt(this, 3, "Terrain name? ");
		}
		break;
	case 3:
		{
			std::string tchar = from -> GetCommandData();
			Terrain t(db, area, tchar[0]);
			if (line.size())
			{
				t.SetName(line);
				t.Save();
			}
			if (t.IsImpassable())
			{
				from -> SetPrompt(this, 4, "Impassable (Y/n)? ");
			}
			else
			{
				from -> SetPrompt(this, 4, "Impassable (y/N)? ");
			}
		}
		break;
	case 4:
		{
			std::string tchar = from -> GetCommandData();
			Terrain t(db, area, tchar[0]);
			if (t.IsImpassable())
			{
				if (line.size() && (line[0] == 'n' || line[0] == 'N'))
				{
					t.SetImpassable(false);
					t.Save();
				}
			}
			else
			{
				if (line.size() && (line[0] == 'y' || line[0] == 'Y'))
				{
					t.SetImpassable();
					t.Save();
				}
			}
			if (t.Exists())
			{
				char slask[100];
				sprintf(slask, "Delay (%.2f sec)? ", t.GetDelay());
				from -> SetPrompt(this, 5, slask);
			}
			else
			{
				from -> SetPrompt(this, 5, "Delay (seconds)? ");
			}
		}
		break;
	case 5: // delay
		{
			if (line.size())
			{
				std::string tchar = from -> GetCommandData();
				Terrain t(db, area, tchar[0]);
				t.SetDelay(atof(line.c_str()));
				t.Save();
			}
			from -> Send("Ok.\n");
			from -> SetPrompt();
		}
		break;
	case 10: // name of new area
		{
			Area area(db, line);
			if (area.Exists())
			{
				from -> Sendf("An area named '%s' already exists.\n", line.c_str());
				from -> SetPrompt();
			}
			else
			{
				from -> SetCommandData(line);
				from -> SetPrompt(this, 11, "New area width (3..19)? ");
			}
		}
		break;
	case 11: // area width
		{
			int width = atoi(line.c_str());
			if (width > 2 && width < 20)
			{
				from -> SetCommandData2(line);
				from -> SetPrompt(this, 12, "New area height (3..19)? ");
			}
			else
			{
				from -> Send("Aborted. Allowed width = 3..19\n");
				from -> SetPrompt();
			}
		}
		break;
	case 12: // area height
		{
			int height = atoi(line.c_str());
			if (height > 2 && height < 20)
			{
				Area narea(db, from -> GetCommandData());
				narea.SetWidth(atoi(from -> GetCommandData2().c_str()));
				narea.SetHeight(height);
				narea.Save();
				narea.VerifyTerrain();
				Chunk nchunk(db, narea, 1000, 1000, 0);
				nchunk.Init(from, 1000, 1000, 0, ' ');
				nchunk.Save();
				int nx = narea.GetWidth() / 2;
				int ny = narea.GetHeight() / 2;
				Chunk chunk(db, area, pl.GetChunk());
				chunk.AddPortal(pl.GetChunkX(),
					pl.GetChunkY(),
					narea, nchunk, nx, ny);
				nchunk.AddPortal(nx, ny, area, chunk,
					pl.GetChunkX(),
					pl.GetChunkY());
				from -> Send("Area created.\n");
				from -> SetPrompt();
			}
			else
			{
				from -> Send("Aborted. Allowed height = 3..19\n");
				from -> SetPrompt();
			}
		}
		break;
	case 20: // race num to edit
		{
			long num = atol(line.c_str());
			if (!num && line.size() == 1 && line[0] == '0')
			{
				from -> SetCommandData(line);
				from -> SetPrompt(this, 21, "New race name: ");
				return;
			}
			Race r(db, num);
			if (r.Exists())
			{
				from -> SetCommandData(line);
				from -> SetPrompt(this, 21, "Name (" + r.GetName() + ")? ");
			}
			else
			{
				from -> Send("Race does not exist\n");
				from -> SetPrompt();
			}
		}
		break;
	case 21: // new race name
		{
			long num = atol(from -> GetCommandData().c_str());
			std::string name;
			if (num)
			{
				Race r(db, num);
				if (line.size())
				{
					r.SetName(line);
					r.Save();
				}
				from -> SetCommandData(r.GetName());
				name = r.GetName();
			}
			else
			if (!line.size())
			{
				from -> Send("Name required.\n");
				from -> SetPrompt();
				return;
			}
			else
			{
				Race r(db, line);
				r.Save();
				from -> SetCommandData(line); // use name
				name = line;
			}
			char slask[100];
			Race r(db, name);
			sprintf(slask, "SightRange (%.1f)? ", r.GetSightRange());
			from -> SetPrompt(this, 22, slask);
		}
		break;
	case 22: // sight range
		{
			Race r(db, from -> GetCommandData());
			if (line.size())
			{
				r.SetSightRange(atof(line.c_str()));
				r.Save();
			}
			char slask[100];
			sprintf(slask, "Hearing (%.1f)? ", r.GetHearing());
			from -> SetPrompt(this, 23, slask);
		}
		break;
	case 23: // hearing
		{
			Race r(db, from -> GetCommandData());
			if (line.size())
			{
				r.SetHearing(atof(line.c_str()));
				r.Save();
			}
			from -> Send("Ok.\n");
			from -> SetPrompt();
		}
		break;
	}
}


