// Editor.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "Editor.h"
#include "InSocket.h"
#include "Player.h"
#include "Area.h"
#include "RoomDescription.h"
#include "Chunk.h"


Editor::Editor(InSocket *from,editor_t type) : Command(), m_from(from), m_type(type), m_line(1)
{
}


Editor::~Editor()
{
}


void Editor::Init()
{
	Database& db = m_from -> GetDatabase();
	switch (m_type)
	{
	case EDIT_ROOM_DESCRIPTION:
		{
			Player pl(db, m_from -> GetAccountName());
			Area area(db, pl.GetArea());
			Chunk chunk(db, area, pl.GetChunk());
			RoomDescription rd(db, chunk, pl.GetChunkX(), pl.GetChunkY());
			if (rd.Exists())
			{
				std::string tmp;
				for (size_t i = 0; i < rd.GetData().size(); i++)
				{
					if (rd.GetData()[i] == 13)
					{
						// ignore
					}
					else
					if (rd.GetData()[i] == 10)
					{
						AddLine(tmp);
						tmp = "";
					}
					else
					{
						tmp += rd.GetData()[i];
					}
				}
				if (tmp.size())
				{
					AddLine(tmp);
				}
			}
		}
		break;
	}
	//
	if (m_line <= m_edit.size())
		m_from -> Send(m_edit[m_line - 1] + "\n");
	SetPrompt(0);
}


void Editor::Save()
{
	Database& db = m_from -> GetDatabase();
	switch (m_type)
	{
	case EDIT_ROOM_DESCRIPTION:
		{
			Player pl(db, m_from -> GetAccountName());
			Area area(db, pl.GetArea());
			Chunk chunk(db, area, pl.GetChunk());
			RoomDescription rd(db, chunk, pl.GetChunkX(), pl.GetChunkY());
			std::string tmp;
			for (size_t i = 0; i < m_edit.size(); i++)
			{
				tmp += m_edit[i] + "\n";
			}
			rd.SetData(tmp);
			rd.Save();
		}
		break;
	}
}


void Editor::SetPrompt(int state)
{
	char slask[100];
	if (m_line <= m_edit.size())
		sprintf(slask, "%03d>", m_line);
	else
		strcpy(slask, "<END>");
	m_from -> SetPrompt(this, state, slask);
}


void Editor::OnLine(InSocket *from,const std::string& line_in,int state)
{
	std::vector<std::string>::iterator it = m_edit.begin();
	{
		size_t i = 1;
		while (i++ < m_line && it != m_edit.end())
			it++;
	}
	std::string line = line_in;
	switch (state)
	{
	case 0:
		if (!line.size())
		{
			if (m_line <= m_edit.size())
				m_line++;
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
			return;
		}
		if (!strcasecmp(line.c_str(), "a"))
			line = m_prev;
		m_prev = line;
		if (!strcasecmp(line.c_str(), "e"))
		{
			m_line = m_edit.size() + 1;
			SetPrompt(0);
		}
		else
		if (!strcasecmp(line.c_str(), "b"))
		{
			m_line = 1;
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
		}
		else
		if (!strcasecmp(line.c_str(), "i"))
		{
			m_from -> SetPrompt(this, 1, "   .");
		}
		else
		if (!strcasecmp(line.c_str(), "d"))
		{
			if (it != m_edit.end())
				m_edit.erase(it);
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
		}
		else
		if (!strcasecmp(line.c_str(), "qu"))
		{
			m_from -> Send("Exit without save\n");
			m_from -> SetPrompt();
			delete this;
		}
		else
		if (!strcasecmp(line.c_str(), "ex"))
		{
			Save();
			m_from -> Send("Ok.\n");
			m_from -> SetPrompt();
			delete this;
		}
		else
		if (!strcasecmp(line.c_str(), "ec"))
		{
			Save();
			m_from -> Send("Ok.\n");
			m_from -> SetPrompt();
			delete this;
		}
		else
		if (!strcasecmp(line.substr(0, 1).c_str(), "s"))
		{
		}
		else
		if (!strcmp(line.substr(0, 1).c_str(), "+"))
		{
			int x = line.substr(1).size() ? atoi(line.substr(1).c_str()) : 1;
			m_line += x;
			if (m_line > m_edit.size())
				m_line = m_edit.size() + 1;
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
		}
		else
		if (!strcmp(line.substr(0, 1).c_str(), "-"))
		{
			int x = line.substr(1).size() ? atoi(line.substr(1).c_str()) : 1;
			m_line -= x;
			if (m_line < 1)
				m_line = 1;
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
		}
		else
		if (isdigit(line[0]))
		{
			m_line = atoi(line.c_str());
			if (m_line < 1)
				m_line = 1;
			if (m_line > m_edit.size())
				m_line = m_edit.size() + 1;
			if (m_line <= m_edit.size())
				m_from -> Send(m_edit[m_line - 1] + "\n");
			SetPrompt(0);
		}
		else
		if (!strcasecmp(line.substr(0, 1).c_str(), "l"))
		{
			for (size_t i = 0; i < m_edit.size(); i++)
			{
				char slask[100];
				sprintf(slask, "%03d>", i + 1);
				m_from -> Send(slask + m_edit[i] + "\n");
			}
			SetPrompt(0);
		}
		else
		if (!strcmp(line.substr(0, 1).c_str(), ">"))
		{
		}
		else
		if (!strcmp(line.substr(0, 1).c_str(), "<"))
		{
		}
		else
		{
			m_from -> Send(
				"&GE   &ng till slutet av texten&n\n"
				"&GB   &ng till brjan av texten&n\n"
				"&GI   &nskjut in rader fre aktuell rad, avsluta med tom rad&n\n"
				"&GD   &nta bort aktuell rad&n\n"
				"&GQU  &navsluta utan att spara&n\n"
				"&GEX  &navsluta och spara&n\n"
				"&GEC  &navsluta, spara, och kompilera&n\n"
				"&GS/xx/yy    &nbyt ut xx mot yy p aktuell rad&n\n"
				"&G+x  &ng fram x rader&n\n"
				"&G-x  &ng bak x rader&n\n"
				"&G#   &ng till rad nummer #&n\n"
				"&GL   &nlista hela texten&n\n"
				"&GL#  &nlista texten frn rad #&n\n"
				"&G>   &nskjut in 2 mellanslag&n\n"
				"&G<   &nta bort 2 mellanslag&n\n"
				"&GA   &nupprepa fregende kommando&n\n");
			m_from -> SendPrompt();
		}
		break;
	case 1: // insert lines
		if (!line.size())
		{
			SetPrompt(0);
			return;
		}
		m_edit.insert(it, line);
		m_line++;
		m_from -> SetPrompt(this, 1, "   .");
		break;
	}
}


