// FutureHandler.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//#include <stdio.h>
#include <libfuture.h>
#include <Parse.h>

#include "FutureHandler.h"
#include "InSocket.h"
#include "Quit.h"
#include "Who.h"
#include "Help.h"
#include "Create.h"
#include "List.h"
#include "Login.h" // ...
#include "Chunk.h"
#include "Look.h"
#include "Set.h"
#include "North.h"
#include "South.h"
#include "East.h"
#include "West.h"
#include "Fill.h"
#include "Player.h"
#include "Save.h"
#include "Left.h"
#include "Right.h"
#include "Forward.h"
#include "Area.h"
#include "Up.h"
#include "Down.h"
#include "Clear.h"
#include "Edge.h"
#include "Enter.h"
#include "Go.h"
#include "Map.h"
#include "Event.h"


FutureHandler::FutureHandler()
:EventHandler()
,m_db("future.db")
,m_login(new Login())
{
	m_command.push_back(new Quit());
	m_command.push_back(new Who());
	m_command.push_back(new Help());
	m_command.push_back(new Create());
	m_command.push_back(new List());
	m_command.push_back(new Look());
	m_command.push_back(new Setcmd());
	m_command.push_back(new North());
	m_command.push_back(new South());
	m_command.push_back(new East());
	m_command.push_back(new West());
	m_command.push_back(new Fill());
	m_command.push_back(new Save());
	m_command.push_back(new Left());
	m_command.push_back(new Right());
	m_command.push_back(new Forward());
	m_command.push_back(new Up());
	m_command.push_back(new Down());
	m_command.push_back(new Clear());
	m_command.push_back(new Edge());
	m_command.push_back(new Enter());
	m_command.push_back(new Go());
	m_command.push_back(new Map());

	// world setup
	Query q(m_db);
	if (!q.get_count("select count(*) from area"))
	{
		Area a(m_db, "World");
		a.SetWidth(15);
		a.SetHeight(9);
		a.Save();
	}
	if (!q.get_count("select count(*) from chunk"))
	{
		Area a(m_db, "World");
		Chunk cc(m_db, a, 1000, 1000, 0);
		cc.Init(NULL, 1000, 1000, 0, ' ');
		cc.Save();
	}
}


FutureHandler::~FutureHandler()
{
	while (m_command.size())
	{
		std::list<Command *>::iterator it = m_command.begin();
		Command *p = *it;
		delete p;
		m_command.erase(it);
	}
	delete m_login;
}


void FutureHandler::Talk(InSocket *from,const std::string& line)
{
	size_t i = 0;
	while (i < line.size() && line[i] == ' ')
		i++;
	std::string does;
	if (line.size() > 2 && line.substr(line.size() - 2) == ":)")
	{
		does = "smiles and says";
		from -> Send("You smile and say '" + line + "&n'.\n");
	}
	else
	if (line.size() > 2 && line.substr(line.size() - 2) == ";)")
	{
		does = "flirts and says";
		from -> Send("You flirt and say '" + line + "&n'.\n");
	}
	else
	if (line.size() > 2 && line.substr(line.size() - 2) == "!!")
	{
		does = "shouts";
		from -> Send("You shout '" + line + "&n'.\n");
	}
	else
	if (line.substr(line.size() - 1) == "?")
	{
		does = "asks";
		from -> Send("You ask '" + line + "&n'.\n");
	}
	else
	if (line.substr(line.size() - 1) == "!")
	{
		does = "exclaims";
		from -> Send("You exclaim '" + line + "&n'.\n");
	}
	else
	{
		does = "says";
		from -> Send("You say '" + line + "&n'.\n");
	}
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	std::string talk = "\n" + pl.GetDisplayName() + " " + does + " '" + line.substr(i) + "&n'.\n";
	for (socket_m::iterator it = m_sockets.begin(); it != m_sockets.end(); it++)
	{
		Socket *p0 = (*it).second;
		InSocket *p = dynamic_cast<InSocket *>(p0);
		if (p && p != from)
		{
			p -> Send(talk);
			p -> SendPrompt();
		}
	}
}


void FutureHandler::GlobalEvent(InSocket *from,const std::string& s)
{
	std::string talk = "\n" + s + "\n";
	for (socket_m::iterator it = m_sockets.begin(); it != m_sockets.end(); it++)
	{
		Socket *p0 = (*it).second;
		InSocket *p = dynamic_cast<InSocket *>(p0);
		if (p && p != from)
		{
			p -> Send(talk);
			p -> SendPrompt();
		}
	}
}


void FutureHandler::DoCommand(InSocket *from,const std::string& cmd,const std::string& params)
{
	Command *match = NULL;
	if (cmd.size() == 1)
	{
		match = Command::GetCommandFromChar(cmd[0]);
	}
	if (match)
	{
		Player pl(from -> GetDatabase(),from -> GetAccountName());
		if (!match -> NeedWiz() || pl.IsWizard())
		{
			Parse pa(params);
			std::string arg = pa.getword();
			std::string complete;
			if (match -> CheckArgument(from, arg, complete))
			{
				match -> Execute(from, complete, pa);
			}
			return;
		}
	}
	int score = 0;
	int count = 0;
	for (std::list<Command *>::iterator it = m_command.begin(); it != m_command.end(); it++)
	{
		Command *p = *it;
		int i = p -> Match(from, cmd, params);
		score = (i > score) ? i : score;
		if (i)
		{
			match = p;
			count++;
		}
		if (score == 2)
		{
			break;
		}
	}
	if (score == 1 && count > 1)
	{
		from -> Send("Ambiguous command.\n");
		for (std::list<Command *>::iterator it = m_command.begin(); it != m_command.end(); it++)
		{
			Command *p = *it;
			if (p -> Match(from, cmd, params) == 1)
			{
				from -> Send(" " + p -> GetCommand() + "\n");
			}
		}
		from -> SetPrompt();
	}
	else
	if (score == 2 || (score == 1 && count == 1))
	{
		Parse pa(params);
		std::string arg = pa.getword();
		std::string complete;
		if (match -> CheckArgument(from, arg, complete))
		{
			match -> Execute(from, complete, pa);
		}
	}
	else
	{
		from -> Send("Huh?\n");
		from -> SetPrompt();
	}
}


void FutureHandler::ShowHelp(InSocket *from,const std::string& params)
{
	std::map<std::string,std::map<std::string,Command *> > mmap;
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	for (std::list<Command *>::iterator it = m_command.begin(); it != m_command.end(); it++)
	{
		Command *cc = *it;
		if (!cc -> NeedWiz() || pl.IsWizard())
		{
			mmap[cc -> Category()][cc -> GetCommand()] = cc;
		}
	}
	for (std::map<std::string,std::map<std::string,Command *> >::iterator it = mmap.begin(); it != mmap.end(); it++)
	{
		std::string category = (*it).first;
		std::map<std::string,Command *>& ref = (*it).second;
		from -> Send("&W" + category + "&n\n");
		for (std::map<std::string,Command *>::iterator it = ref.begin(); it != ref.end(); it++)
		{
			std::string command = (*it).first;
			Command *cc = (*it).second;
			strbool_m& ref = cc -> GetArguments();
			std::string charstr;
			std::string argstr;
			std::string descr;
			if (cc -> GetCommandChar())
			{
				charstr = " &g(&y";
				charstr += cc -> GetCommandChar();
				charstr += "&g)&n";
			}
			for (strbool_m::iterator it = ref.begin(); it != ref.end(); it++)
			{
				std::string arg = (*it).first;
				bool need_wiz = (*it).second;
				if (!need_wiz || pl.IsWizard())
				{
					if (argstr.size())
						argstr += "&g|&y" + arg;
					else
						argstr = " &g[&y" + arg;
				}
			}
			if (argstr.size())
				argstr += "&g]&n";
			if (cc -> Description().size())
				descr = "  - &M" + cc -> Description();
			from -> Send("  &G" + command + charstr + argstr + descr + "&n\n");
		}
	}
}


void FutureHandler::Count(const std::string& event)
{
	db::Eventcount ec(m_db, event);
	ec.event_name = event;
	ec.count++;
	ec.save();
}


std::string FutureHandler::GetDateTime()
{
	time_t t = time(NULL);
	struct tm *tp = localtime(&t);
	char dt[40];
	sprintf(dt, "%d-%02d-%02d %02d:%02d:%02d", tp -> tm_year + 1900, tp -> tm_mon + 1,
		tp -> tm_mday, tp -> tm_hour, tp -> tm_min, tp -> tm_sec);
	return dt;
}


Command *FutureHandler::GetCommand(const std::string& x)
{
	for (std::list<Command *>::iterator it = m_command.begin(); it != m_command.end(); it++)
	{
		Command *cc = *it;
		if (cc -> GetCommand() == x)
			return cc;
	}
	return NULL;
}


