// InSocket.h
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _INSOCKET_H
#define _INSOCKET_H

#include <TcpSocket.h>
#include <SocketHandler.h>
#include <IEventOwner.h>

/* strings */

#define MXP_BEG "\x03"    /* becomes < */
#define MXP_END "\x04"    /* becomes > */
#define MXP_AMP "\x05"    /* becomes & */

/* characters */

#define MXP_BEGc '\x03'    /* becomes < */
#define MXP_ENDc '\x04'    /* becomes > */
#define MXP_AMPc '\x05'    /* becomes & */

/* constructs an MXP tag with < and > around it */

#define MXPTAG(arg) MXP_BEG arg MXP_END

#define ESC "\x1B"  /* esc character */

#define MXPMODE(arg) ESC "[" #arg "z"

/* flags for show_list_to_char */

enum 
{
	eItemNothing,   /* item is not readily accessible */
	eItemGet,     /* item on ground */
	eItemDrop,    /* item in inventory */
	eItemBid     /* auction item */
};


class Command;
class Player;
class FutureHandler;
class Database;
struct EVENT;

/** Connection.
	\ingroup system */
class InSocket : public TcpSocket,public IEventOwner
{
public:
	InSocket(SocketHandler&);
	~InSocket();

	void OnDelete();
	void OnAccept();
	void OnLine(const std::string& );
	void ReadLine();

	void Send(const std::string&);
	void Sendf(char *format, ...);
	void SendPrompt();

	void SetPrompt(const std::string& = "",bool colorize = true);
	void SetPrompt(Command *,int state,const std::string& ,bool colorize = true);
	void SetCommandData(const std::string& x) { m_command_data = x; }
	const std::string& GetCommandData() { return m_command_data; }
	void SetCommandData2(const std::string& x) { m_command_data2 = x; }
	const std::string& GetCommandData2() { return m_command_data2; }

	bool UseMxp() { return m_b_use_mxp; }
	Database& GetDatabase();

	void SetAccountName(const std::string& x) { m_account_name = x; }
	const std::string& GetAccountName() { return m_account_name; }

	void OnEvent(int id);
	int AddEvent(long sec,long usec);

	int GetMoveEvent() { return m_move_event; }
	void SetMoveEvent(long sec,long usec);
	void BufferMove(const std::string& );
	void CancelMoves();

	void GlobalEvent(const std::string&);
	void Count(const std::string& );

private:
	void send_mxp_defines();
	void write_to_buffer(const char *);
	void write_to_buffer(const unsigned char *);
	std::string m_prompt;
	Command *m_active_command;
	int m_command_state;
	std::string m_command_data; // 1
	std::string m_command_data2;
	bool m_colorize_prompt;
	bool m_b_use_mxp;
	std::string m_account_name;
	int m_move_event;
	std::list<std::string> m_move_buffer;
};




#endif // _INSOCKET_H
