// Login.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <Utility.h>

#include "Login.h"
#include "InSocket.h"
#include "Chunk.h"
#include "Player.h"
#include "Race.h"


void Login::Execute(InSocket *,const std::string& params,Parse& pa)
{
}


void Login::OnLine(InSocket *from,const std::string& line,int state)
{
	Database& db = from -> GetDatabase();
	switch (state)
	{
	case 0: // name
		if (line.size())
		{
			Player pl(db, line);
			from -> SetCommandData(line);
			if (pl.Exists())
			{
				// %! stealth (echo off) password
				from -> SetPrompt(this, 1, "Password? ");
			}
			else
			{
				from -> Send("Name does not exist.\n");
				from -> SetPrompt(this, 2, "Create a new player named '" + line + "' (y/N)? ");
			}
		}
		else
		{
			from -> Send("Name required (bye).\n");
			from -> SetCloseAndDelete();
		}
		break;
	case 1: // password
		{
			Player pl(db, from -> GetCommandData());
			if (!strcmp(pl.GetPasswd().c_str(), line.c_str()))
			{
				from -> Count("Number of successful logins");
				from -> GlobalEvent(pl.GetDisplayName() + " enters the game.");
				pl.VerifyPos();
				pl.LoadSteps();
				from -> SetAccountName(pl.GetName());
				from -> Send("Welcome, " + pl.GetDisplayName() + "!\n");
				if (!pl.GetRace())
				{
					int n = Race::DisplayRaces(from);
					from -> SetPrompt(this, 6, "Select a race (1-" + Utility::l2string(n) + ")? ");
				}
				else
				{
					from -> SetPrompt();
				}
			}
			else
			{
				from -> Send("Name and/or password incorrect.\n");
				from -> SetPrompt(this, 0, "Name, please? ");
			}
		}
		break;
	case 2: // create (y/N)?
		if (line.size() && (line[0] == 'y' || line[0] == 'Y'))
		{
			from -> SetPrompt(this, 3, "Enter a password: ");
		}
		else
		{
			from -> SetPrompt(this, 0, "Name, please? ");
		}
		break;
	case 3: // enter new password
		if (line.size() < 4)
		{
			from -> Send("Enter at least 4 characters.\n");
			from -> SetPrompt(this, 3, "Enter a password: ");
		}
		else
		{
			from -> SetCommandData2(line);
			from -> SetPrompt(this, 4, "Repeat password: ");
		}
		break;
	case 4: // repeat password
		if (!strcmp(line.c_str(), from -> GetCommandData2().c_str()))
		{
			from -> Count("Number of created logins");
			// wizard set by Player constructor
			// first created is wizard
			Player pl(db, from -> GetCommandData());
			pl.SetName(from -> GetCommandData());
			pl.SetPasswd(line);
			pl.SetDisplayName(from -> GetCommandData());
			pl.VerifyPos();
			pl.Save();
			from -> SetAccountName(from -> GetCommandData());
			from -> SetPrompt(this, 5, "Enter your email: ");
		}
		else
		{
			from -> Send("Password mismatch, please try again.\n");
			from -> SetPrompt(this, 3, "Enter a password: ");
		}
		break;
	case 5: // email
		if (line.size())
		{
			Player pl(db, from -> GetAccountName());
			pl.SetEmail(line);
			pl.Save();
		}
		from -> GlobalEvent("New player " + from -> GetCommandData() + " enters the game.");
		from -> Send("Welcome, new player " + from -> GetCommandData() + "!\n");
		Race::DisplayRaces(from);
		from -> SetPrompt(this, 6, "Select a race: ");
		break;
	case 6: // race
		{
			Player pl(db, from -> GetAccountName());
			long num = atol(line.c_str());
			Race r(db, num);
			if (r.Exists())
			{
				pl.SetRace(num);
				pl.Save();
				//
				from -> SetPrompt();
			}
			else
			if (pl.IsWizard())
			{
				from -> Send("Skipping race selection, for now..\n");
				from -> SetPrompt();
			}
			else
			{
				Race::DisplayRaces(from);
				from -> SetPrompt(this, 6, "Select a race: ");
			}
		}
		break;
	}
}


