// Look.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <math.h>

#include "Look.h"
#include "InSocket.h"
#include "Chunk.h"
#include "Player.h"
#include "Terrain.h"
#include "Area.h"
#include "Steps.h"


void Look::Execute(InSocket *from,const std::string& params,Parse& pa)
{
	LookMap(from);
	Show(from);
	from -> SetPrompt();
}


void Look::LookMap(InSocket *from)
{
	Player pl(from -> GetDatabase(),from -> GetAccountName());
	if (pl.IsWizView() && pl.IsWizard())
	{
		WizMap(from);
	}
	else
	{
		NormalMap(from);
	}
}


void Look::WizMap(InSocket *from)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());

	std::string mapstr;
	std::string colorstr;
	for (int y = 0; y < area.GetHeight(); y++)
	{
		for (int x = 0; x < area.GetWidth(); x++)
		{
			if (x == pl.GetChunkX() && y == pl.GetChunkY())
			{
				mapstr += "&nA";
				colorstr = "&n";
			}
			else
			{
				Terrain t(db, area, chunk.Get(x, y));
				if (t.Exists())
				{
					if (t.GetColor() != colorstr)
					{
						mapstr += t.GetColor();
						colorstr = t.GetColor();
					}
					mapstr += t.GetChar();
				}
				else
				{
					mapstr += ' ';
				}
			}
		}
		mapstr += "&n\n";
		colorstr = "&n";
	}
	from -> Send(mapstr);
	//Show(from);
}


void Look::NormalMap(InSocket *from)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());
	int cx = pl.GetChunkX();
	int cy = pl.GetChunkY();
	double sightrange = pl.SightRange();
	int xr = (int)(3 * sightrange / 2) + 1;
	int yr = (int)sightrange + 1;

	long xcoord = pl.GetCoordX(area, chunk);
	long ycoord = pl.GetCoordY(area, chunk);
	Steps st(db, pl, area, chunk.GetZ(), ycoord);
	st.AddCoord(xcoord);

	std::string mapstr;
	std::string colorstr;

	for (int y = -yr; y <= yr; y++)
	{
		for (int x = -xr; x <= xr; x++)
		{
			int xtmp = 2 * x / 3;
			if (sqrt(xtmp * xtmp + y * y) >= sightrange)
			{
				mapstr += "&n ";
				colorstr = "&n";
			}
			else
			if (cx + x == pl.GetChunkX() && cy + y == pl.GetChunkY())
			{
				mapstr += "&nA";
				colorstr = "&n";
			}
			else
			{
				bool can_see = check_facing(x, y, pl.GetFacing());
				getmap(db, area, chunk, cx + x, cy + y, mapstr, colorstr, can_see);
			}
		}
		mapstr += "&n\n";
		colorstr = "&n";
	}

	from -> Send(mapstr);
	//Show(from);
}


void Look::getmap(Database& db,Area& area,Chunk& chunk,int x,int y,std::string& mapstr,std::string& colorstr,bool color)
{
	if (x >= 0 && x < area.GetWidth() && y >= 0 && y < area.GetHeight())
	{
		Terrain t(db, area, chunk.Get(x, y));
		if (t.Exists())
		{
			if (!color)
			{
				if (colorstr != "&n")
				{
					mapstr += "&n";
					colorstr = "&n";
				}
			}
			else
			if (t.GetColor() != colorstr)
			{
				mapstr += t.GetColor();
				colorstr = t.GetColor();
			}
			mapstr += t.GetChar();
		}
		else
		{
			if (colorstr != "&n")
			{
				mapstr += "&n";
				colorstr = "&n";
			}
			mapstr += ' ';
		}
	}
	else
	{
		int chunk_x = chunk.GetX();
		int chunk_y = chunk.GetY();
		int chunk_z = chunk.GetZ();
		while (x < 0)
		{
			chunk_x -= 1; // west
			x += area.GetWidth();
		}
		while (x >= area.GetWidth())
		{
			chunk_x += 1; // east
			x -= area.GetWidth();
		}
		while (y < 0)
		{
			chunk_y += 1; // north
			y += area.GetHeight();
		}
		while (y >= area.GetHeight())
		{
			chunk_y -= 1; // south
			y -= area.GetHeight();
		}
		Chunk cc(db, area, chunk_x, chunk_y, chunk_z);
		Terrain t(db, area, cc.Get(x, y));
		if (cc.Exists() && t.Exists())
		{
			if (!color)
			{
				if (colorstr != "&n")
				{
					mapstr += "&n";
					colorstr = "&n";
				}
			}
			else
			if (t.GetColor() != colorstr)
			{
				mapstr += t.GetColor();
				colorstr = t.GetColor();
			}
			mapstr += t.GetChar();
		}
		else
		{
			if (colorstr != "&n")
			{
				mapstr += "&n";
				colorstr = "&n";
			}
			mapstr += ' ';
		}
	}
}


void Look::Show(InSocket *from)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());
	Terrain t(db, area, chunk.Get(pl.GetChunkX(), pl.GetChunkY()));
	if (t.Exists())
		from -> Send(t.GetColor() + t.GetName() + "&n\n");
	chunk.Show(from);
}


bool Look::check_facing(int x,int y,int facing)
{
	if (x > -2 && x < 2 && y > -2 && y < 2)
		return true;
	/** 1-north, 2-south, 3-east, 4-west */
	switch (facing)
	{
	case 1:
		if (y <= 0)
			return true;
		break;
	case 2:
		if (y >= 0)
			return true;
		break;
	case 3:
		if (x >= 0)
			return true;
		break;
	case 4:
		if (x <= 0)
			return true;
		break;
	}
	return false;
}


