// Map.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "Map.h"
#include "InSocket.h"
#include "Player.h"
#include "Area.h"
#include "Chunk.h"
#include "Terrain.h"


void Map::Execute(InSocket *from,const std::string& params,Parse&)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());
	int xmin,xmax,ymin,ymax;
	std::string colorstr;
	area.GetAreaBorders(chunk.GetZ(),xmin,xmax,ymin,ymax);

	for (int ch_y = ymax; ch_y >= ymin; ch_y--)
	{
		for (int y = 0; y < area.GetHeight(); y++)
		{
			for (int ch_x = xmin; ch_x <= xmax; ch_x++)
			{
				Chunk cc(db, area, ch_x, ch_y, chunk.GetZ() );
				for (int x = 0; x < area.GetWidth(); x++)
				{
					long xcoord = ch_x * area.GetWidth() + x;
					long ycoord = ch_y * area.GetHeight() + area.GetHeight() - y - 1;
					if (pl.Seen(area, xcoord, ycoord, chunk.GetZ()))
					{
					if (cc.Exists())
					{
						std::string tmp;
						tmp += cc.Get(x,y);
						Terrain t(db, area, tmp[0]);
						if (t.GetColor() != colorstr)
						{
							from -> Send("&n" + t.GetColor());
							colorstr = t.GetColor();
						}
						from -> Send(tmp);
					}
					else
					{
						from -> Send("&n ");
						colorstr = "&n";
					}
					}
					else
					{
						from -> Send("&n ");
						colorstr = "&n";
					}
				}
			}
			from -> Send("&n\n");
			colorstr = "&n";
		}
	}
	from -> SetPrompt();
}


