// Move.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "Move.h"
#include "InSocket.h"
#include "Player.h"
#include "Chunk.h"
#include "Look.h"
#include "Terrain.h"
#include "Area.h"


void Move::move_to(InSocket *from,Chunk& north,int ny_x,int ny_y,bool hide_map)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Terrain t(db, area, north.Get(ny_x, ny_y));
	if ((t.Exists() && !t.IsImpassable()) || pl.IsWizard())
	{
		pl.SetChunk(north.GetNum());
		pl.SetChunkX(ny_x);
		pl.SetChunkY(ny_y);
		/** 1-north, 2-south, 3-east, 4-west */
		if (m_loc_y < 0)
			pl.SetFacing(1);
		if (m_loc_y > 0)
			pl.SetFacing(2);
		if (m_loc_x > 0)
			pl.SetFacing(3);
		if (m_loc_x < 0)
			pl.SetFacing(4);
		if (!hide_map)
			Look::LookMap(from);
		Look::Show(from);
		from -> SetMoveEvent(0, 1000);
	}
	else
	{
		from -> Send("Impassable terrain ahead.\n");
		from -> CancelMoves();
	}
	from -> SetPrompt();
}


void Move::try_move(InSocket *from, int ch_x, int ch_y, int ch_z, bool hide_map)
{
	Database& db = from -> GetDatabase();
	Player pl(db, from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());

	int ny_x = pl.GetChunkX() + m_loc_x;
	int ny_y = pl.GetChunkY() + m_loc_y;
	if (!ch_z && ny_x >= 0 && ny_x < area.GetWidth() && ny_y >= 0 && ny_y < area.GetHeight())
	{
		move_to(from, chunk, ny_x, ny_y, hide_map);
	}
	else
	{
		if (ch_z && !pl.IsWizard()) // up/down check
		{
			Terrain t(db, area, chunk.Get(ny_x, ny_y));
			if ((ch_z == 1 && t.GetChar() != 'U') || (ch_z == -1 && t.GetChar() != 'D'))
			{
				from -> Send("No exit that way.\n");
				from -> CancelMoves();
				from -> SetPrompt();
				return;
			}
		}
		Chunk north(db, area, chunk.GetX() + ch_x, chunk.GetY() + ch_y, chunk.GetZ() + ch_z);
		if (north.Exists())
		{
			while (ny_x < 0)
				ny_x += area.GetWidth();
			while (ny_y < 0)
				ny_y += area.GetHeight();
			while (ny_x >= area.GetWidth())
				ny_x -= area.GetWidth();
			while (ny_y >= area.GetHeight())
				ny_y -= area.GetHeight();
			move_to(from, north, ny_x, ny_y, hide_map);
		}
		else
		if (pl.IsWizard())
		{
			from -> SetPrompt(this, 0, "Create a new chunk of land (y/N)? ");
			from -> CancelMoves();
		}
		else
		{
			from -> Send("End of the world.\n");
			from -> CancelMoves();
			from -> SetPrompt();
		}
	}
}


void Move::try_create(InSocket *from,const std::string& line,int ch_x,int ch_y,int ch_z)
{
	Database& db = from -> GetDatabase();
	Player pl(from -> GetDatabase(), from -> GetAccountName());
	Area area(db, pl.GetArea());
	Chunk chunk(db, area, pl.GetChunk());

	int ny_x = pl.GetChunkX() + m_loc_x;
	int ny_y = pl.GetChunkY() + m_loc_y;
	while (ny_x < 0)
		ny_x += area.GetWidth();
	while (ny_y < 0)
		ny_y += area.GetHeight();
	while (ny_x >= area.GetWidth())
		ny_x -= area.GetWidth();
	while (ny_y >= area.GetHeight())
		ny_y -= area.GetHeight();
	if (line.size() && (line[0] == 'y' || line[0] == 'Y'))
	{
		Chunk north(db, area, chunk.GetX() + ch_x, chunk.GetY() + ch_y, chunk.GetZ() + ch_z);
		if (!north.Exists())
		{
			north.Init(from, chunk.GetX() + ch_x, chunk.GetY() + ch_y, chunk.GetZ() + ch_z, ' ');
			north.Save();
		}
		if (ch_z == 1) // create up
		{
			chunk.Set(ny_x, ny_y, 'U');
			north.Set(ny_x, ny_y, 'D');
			chunk.Save();
			north.Save();
		}
		else
		if (ch_z == -1) // create down
		{
			chunk.Set(ny_x, ny_y, 'D');
			north.Set(ny_x, ny_y, 'U');
			chunk.Save();
			north.Save();
		}
		pl.SetChunk(north.GetNum());
		pl.SetChunkX(ny_x);
		pl.SetChunkY(ny_y);
		Look::LookMap(from);
		Look::Show(from);
	}
	from -> SetPrompt();
}


