// Player.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <libfuture.h>
#include <Utility.h>
#include <Parse.h>
#include <math.h>

#include "Player.h"
#include "Chunk.h"
#include "Area.h"
#include "Steps.h"
#include "Race.h"


// statics
accounts_t Player::m_accounts;


Player::Player(Database& db,const std::string& name) : Object(db)
, m_account(Player::GetAccount(db, name))
{
}


Player::~Player()
{
}


db::Account *Player::GetAccount(Database& db,const std::string& name)
{
	db::Account *p = m_accounts[name];
	if (p != NULL)
		return p;
	p = new db::Account(db, name);
	p -> name = name;
	if (!p -> num)
	{
		Query q(db);
		long nr = q.get_count("select count(*) from account");
		p -> wizard = !nr;
	}
	m_accounts[name] = p;
	return p;
}


void Player::Save()
{
	if (m_account)
		m_account -> save();
}


bool Player::Exists()
{
	if (m_account && m_account -> num)
		return true;
	return false;
}


std::string Player::GetName()
{
	return m_account ? m_account -> name: "";
}


bool Player::IsWizard()
{
	return m_account ? (m_account -> wizard ? true : false) : false;
}


std::string Player::GetPasswd()
{
	return m_account -> passwd;
}


std::string Player::GetDisplayName()
{
	return m_account -> display_name + "&n";
}


void Player::VerifyPos()
{
	if (!m_account -> area)
	{
		Area area(m_db, "World");
		Chunk chunk(m_db, area, 1000, 1000, 0); // default starting point
		m_account -> area = area.GetNum();
		m_account -> chunk = chunk.GetNum();
		m_account -> chunkx = area.GetWidth() / 2;
		m_account -> chunky = area.GetHeight() / 2;
		m_account -> save();
	}
	if (!m_account -> facing)
	{
		m_account -> facing = 1;
		Save();
	}
}


void Player::SetName(const std::string& x)
{
	m_account -> name = x;
}


void Player::SetPasswd(const std::string& x)
{
	m_account -> passwd = x;
}


void Player::SetWizard(bool x)
{
	m_account -> wizard = x ? 1 : 0;
}


void Player::SetDisplayName(const std::string& x)
{
	m_account -> display_name = x;
}


void Player::SetEmail(const std::string& x)
{
	m_account -> email = x;
}


long Player::GetArea()
{
	return m_account -> area;
}


long Player::GetChunk()
{
	return m_account -> chunk;
}


long Player::GetChunkX()
{
	return m_account -> chunkx;
}


long Player::GetChunkY()
{
	return m_account -> chunky;
}


void Player::SetChunkX(int x)
{
	m_account -> chunkx = x;
}


void Player::SetChunkY(int y)
{
	m_account -> chunky = y;
}


void Player::SetArea(long x)
{
	m_account -> area = x;
}


void Player::SetChunk(long x)
{
	m_account -> chunk = x;
}


long Player::GetNum()
{
	return m_account -> num;
}


int Player::GetFacing()
{
	return m_account -> facing;
}


void Player::SetFacing(int x)
{
	m_account -> facing = x;
}


void Player::SetWizView(bool x)
{
	m_account -> wizview = x ? 1 : 0;
}


bool Player::IsWizView()
{
	return (m_account -> wizview && m_account -> wizard) ? true : false;
}


long Player::GetCoordX(Area& area,Chunk& chunk)
{
	return chunk.GetX() * area.GetWidth() + GetChunkX();
}


long Player::GetCoordY(Area& area,Chunk& chunk)
{
	return chunk.GetY() * area.GetHeight() + area.GetHeight() - GetChunkY() - 1;
}


bool Player::Seen(Area& area,long xcoord,long ycoord,int ch_z)
{
	for (int y = 0; y < SightRange() + 1; y++)
	{
		db::Steps *st;
		if (y)
		{
			st = Steps::GetStep(*this, area, ch_z, ycoord - y);
			if (st)
			{
				Parse pa(st -> xcoords, ":");
				std::string tmp = pa.getword();
				while (tmp.size())
				{
					long x = atol(tmp.c_str());
					long x0 = x - xcoord;
					long y0 = st -> ycoord - ycoord;
					long xtmp = 2 * x0 / 3;
					if (sqrt(xtmp * xtmp + y0 * y0) < SightRange())
					{
						return true;
					}
					//
					tmp = pa.getword();
				}
			}
			st = Steps::GetStep(*this, area, ch_z, ycoord + y);
			if (st)
			{
				Parse pa(st -> xcoords, ":");
				std::string tmp = pa.getword();
				while (tmp.size())
				{
					long x = atol(tmp.c_str());
					long x0 = x - xcoord;
					long y0 = st -> ycoord - ycoord;
					long xtmp = 2 * x0 / 3;
					if (sqrt(xtmp * xtmp + y0 * y0) < SightRange())
					{
						return true;
					}
					//
					tmp = pa.getword();
				}
			}
		}
		else
		{
			st = Steps::GetStep(*this, area, ch_z, ycoord);
			if (st)
			{
				Parse pa(st -> xcoords, ":");
				std::string tmp = pa.getword();
				while (tmp.size())
				{
					long x = atol(tmp.c_str());
					long x0 = x - xcoord;
					long y0 = st -> ycoord - ycoord;
					long xtmp = 2 * x0 / 3;
					if (sqrt(xtmp * xtmp + y0 * y0) < SightRange())
					{
						return true;
					}
					//
					tmp = pa.getword();
				}
			}
		}
	}
	return false;
}


double Player::SightRange()
{
	Race r(m_db, m_account -> race);
	if (r.Exists())
		return r.GetSightRange();
	return 3.5;
}


void Player::LoadSteps()
{
	Query q(m_db);
	q.get_result("select * from steps where player=" + Utility::l2string(GetNum()));
	while (q.fetch_row())
	{
		db::Steps st(&m_db, &q);
		Area area(m_db, st.area);
		Steps s(m_db, *this, area, st.zcoord, st.ycoord);
	}
	q.free_result();
}


void Player::SetRace(long x)
{
	m_account -> race = x;
}


long Player::GetRace()
{
	return m_account -> race;
}


