// Player.h
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _PLAYER_H
#define _PLAYER_H

#include <string>
#include <map>
#include "Object.h"


namespace db { class Account; }
class Database;
class Area;
class Chunk;

typedef std::map<std::string,db::Account *> accounts_t;

/** Database table Player
	\ingroup db */
class Player : public Object
{
public:
	Player(Database&, const std::string& name);
	~Player();

	std::string GetName();
	bool IsWizard();
	std::string GetPasswd();
	std::string GetDisplayName();
	void SetName(const std::string&);
	void SetPasswd(const std::string& );
	void SetWizard(bool);
	void SetDisplayName(const std::string&);
	void SetEmail(const std::string&);
	long GetArea();
	long GetChunk();
	long GetChunkX();
	long GetChunkY();
	void SetChunkX(int);
	void SetChunkY(int);
	void SetArea(long);
	void SetChunk(long);
	long GetNum();
	/** 1-north, 2-south, 3-east, 4-west */
	int GetFacing();
	/** 1-north, 2-south, 3-east, 4-west */
	void SetFacing(int);
	void SetRace(long);
	long GetRace();

	void VerifyPos();

	void Save();
	bool Exists();

static	db::Account *GetAccount(Database& db,const std::string& name);

	void SetWizView(bool = true);
	bool IsWizView();

	long GetCoordX(Area&,Chunk&);
	long GetCoordY(Area&,Chunk&);
	bool Seen(Area&,long xcoord,long ycoord,int ch_z);
	double SightRange();
	void LoadSteps();

private:
	db::Account *m_account;
static	accounts_t m_accounts;
};




#endif // _PLAYER_H
