// Race.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <libfuture.h>
#include <Utility.h>

#include "Race.h"
#include "InSocket.h"


// statics
races_t Race::m_cache;


Race::Race(Database& db,long num)
:Object(db)
,m_race(GetRace(db, num))
{
}


Race::Race(Database& db,const std::string& name)
:Object(db)
,m_race(GetRace(db, name))
{
}


Race::~Race()
{
}


db::Race *Race::GetRace(Database& db,long num)
{
	for (std::map<std::string,db::Race *>::iterator it = m_cache.begin(); it != m_cache.end(); it++)
	{
		db::Race *p = (*it).second;
		if (p -> num == num)
			return p;
	}
	db::Race *p = new db::Race(db, num);
	if (p -> num)
	{
		m_cache[p -> name] = p;
		return p;
	}
	delete p;
	return NULL; // bang
}


db::Race *Race::GetRace(Database& db,const std::string& name)
{
	db::Race *p = m_cache[name];
	if (p)
		return p;
	p = new db::Race(db, name);
	p -> name = name;
	m_cache[name] = p;
	return p;
}


bool Race::Exists()
{
	if (m_race && m_race -> num)
		return true;
	return false;
}


void Race::Save()
{
	m_race -> save();
}


long Race::GetNum()
{
	return m_race -> num;
}


void Race::SetName(const std::string& x)
{
	for (std::map<std::string,db::Race *>::iterator it = m_cache.begin(); it != m_cache.end(); it++)
	{
		std::string name = (*it).first;
		if (name == m_race -> name)
		{
			m_cache.erase(it);
			break;
		}
	}
	m_race -> name = x;
	m_cache[x] = m_race;
}


const std::string& Race::GetName()
{
	return m_race -> name;
}


void Race::SetSightRange(double x)
{
	m_race -> sightrange = x;
}


double Race::GetSightRange()
{
	return m_race -> sightrange;
}


void Race::SetHearing(double x)
{
	m_race -> hearing = x;
}


double Race::GetHearing()
{
	return m_race -> hearing;
}


int Race::DisplayRaces(InSocket *from)
{
	Database& db = from -> GetDatabase();
	Query q(db);
	int n = 0;
	q.get_result("select * from race order by name");
	while (q.fetch_row())
	{
		db::Race r(&db, &q);
		from -> Send(Utility::l2string(r.num) + ") " + r.name + "\n");
		n = MAX(n, r.num);
	}
	q.free_result();
	return n;
}


