// Terrain.cpp
// released 2006-09-25
/*
Copyright (C) 2006  Anders Hedstrom (grymse@alhem.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//#include <stdio.h>
#include <libfuture.h>

#include "Terrain.h"
#include "Area.h"


// statics
terrains_t Terrain::m_terrains;


Terrain::Terrain(Database& db,Area& area,char tchar)
:Object(db)
,m_area(area)
{
	m_terrain = Terrain::GetTerrain(db, area, tchar);
	m_terrain -> area = area.GetNum();
	m_terrain -> tchar = tchar;
}


Terrain::~Terrain()
{
}


db::Terrain *Terrain::GetTerrain(Database& db,Area& area,char tchar)
{
	std::map<char,db::Terrain *>& ref = m_terrains[area.GetNum()];
	db::Terrain *p = ref[tchar];
	if (p != NULL)
		return p;
	std::string tmp;
	tmp += tchar;
	p = new db::Terrain(db, area.GetNum(), tmp);
	p -> area = area.GetNum();
	p -> tchar = tchar;
	ref[tchar] = p;
	return p;
}


bool Terrain::Exists()
{
	if (m_terrain && m_terrain -> num)
		return true;
	return false;
}


const std::string& Terrain::GetColor()
{
	return m_terrain -> tcolor;
}


char Terrain::GetChar()
{
	return m_terrain -> tchar[0];
}


void Terrain::Save()
{
	m_terrain -> save();
}


void Terrain::SetChar(char tchar)
{
	m_terrain -> tchar = tchar;
}


void Terrain::SetColor(const std::string& x)
{
	m_terrain -> tcolor = x;
}


const std::string& Terrain::GetName()
{
	return m_terrain -> name;
}


void Terrain::SetName(const std::string& x)
{
	m_terrain -> name = x;
}


bool Terrain::IsImpassable()
{
	return m_terrain -> impassable ? true : false;
}


void Terrain::SetImpassable(bool x)
{
	m_terrain -> impassable = x ? 1 : 0;
}


long Terrain::GetNum()
{
	return m_terrain -> num;
}


double Terrain::GetDelay()
{
	return m_terrain -> delay;
}


void Terrain::SetDelay(double d)
{
	m_terrain -> delay = d;
}


